# -*- coding:utf-8 -*-
import sys
import requests
from .public import *
from requests.adapters import HTTPAdapter


@异常处理返回类型逻辑型
def 网页_取外网IP(返回地区=False):
    '如果设置返回地区则返回两个参数,ip,地区'
    header = {"User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.152 Safari/537.36"}
    requests.packages.urllib3.disable_warnings()
    try:
        源码 = requests.get("http://pv.sohu.com/cityjson", verify=False, headers=header)
        结果 = eval(源码.text[19:-1])
        if 返回地区 == False:
            return 结果['cip']
        return 结果['cip'], 结果['cname']
    except:
        pass

    try:
        源码 = requests.get("https://ipservice.ws.126.net/locate/api/getLocByIp?callback=bowlder.cb._2", verify=False, headers=header)
        结果 = eval(源码.text[14:-1])
        if 返回地区 == False:
            return 结果['result']['ip']
        return 结果['result']['ip'], "{} {} {}".format(结果['result']['country'] , 结果['result']['province'] , 结果['result']['city'])
    except:
        pass

    try:
        源码 = requests.get("https://api.bilibili.com/x/web-interface/zone?jsonp=jsonp", verify=False, headers=header)
        结果 = eval(源码.text)
        if 返回地区 == False:
            return 结果['data']['addr']
        return 结果['data']['addr'], "{} {} {}".format(结果['data']['country'] , 结果['data']['province'] , 结果['data']['city'])
    except:
        if 返回地区 == False:
            return ''
        return '',''



@异常处理返回类型逻辑型
def 网页_COOKIE合并更新(原COOKIE, 新COOKIE):
    '传入的Cookie可以是文本型也可以是字典,返回更新后的COOKIE,字典型'
    最新Cookie = {}
    临时Cookie = {}
    if type(原COOKIE) == str:
        if 原COOKIE.find(";") == -1:
            名称 = 原COOKIE[0:原COOKIE.find("=")].strip(' ')
            值 = 原COOKIE[原COOKIE.rfind(名称) + len(名称) + 1:len(原COOKIE)].strip(' ')
            if 名称 and 值:
                最新Cookie = {名称: 值}
        else:
            cookie数组 = cookie.split(';')
            for x in cookie数组:
                名称 = x[0:x.find("=")].strip(' ')
                值 = x[原COOKIE.rfind(名称) + len(名称) + 1:len(x)].strip(' ')
                if 名称 and 值:
                    最新Cookie[名称] = 值
    else:
        最新Cookie = 原COOKIE

    if type(新COOKIE) == str:
        if 新COOKIE.find(";") == -1:
            名称 = 新COOKIE[0:新COOKIE.find("=")].strip(' ')
            值 = 新COOKIE[新COOKIE.rfind(名称) + len(名称) + 1:len(新COOKIE)].strip(' ')
            if 名称 and 值:
                临时Cookie = {名称: 值}
        else:
            cookie数组 = cookie.split(';')
            for x in cookie数组:
                名称 = x[0:x.find("=")].strip(' ')
                值 = x[新COOKIE.rfind(名称) + len(名称) + 1:len(x)].strip(' ')
                if 名称 and 值:
                    临时Cookie[名称] = 值
    else:
        临时Cookie = 新COOKIE

    for x in 临时Cookie:
        最新Cookie[x] = 临时Cookie[x]

    return 最新Cookie


class 网页返回类型:
    def __init__(self):
        self.源码 = ''
        self.字节集 = b'' #返回字节集,如图片,视频等文件需要
        self.cookie = {}
        self.协议头 = {}
        self.状态码 = 0
        self.原对象 = None
        self.json = {}

class 网页_访问_会话:
    'requests.session()'

    def __init__(self,重试次数=0):
        self._requests = requests.session()
        if 重试次数:
            self._requests.mount('http://', HTTPAdapter(max_retries=重试次数))
            self._requests.mount('https://', HTTPAdapter(max_retries=重试次数))

    @异常处理返回类型逻辑型
    def 网页_访问(self,url, 方式=0, 参数='', cookie='', 协议头={}, 允许重定向=True, 代理地址=None, 编码=None, 证书验证=False, 上传文件=None, 补全协议头=True,json={},连接超时=15, 读取超时=15):
        """
        :param url: 链接,能自动补全htpp,去除首尾空格
        :param 方式: 0.get 1.post 2.put 3.delete 4.head 5.options
        :param 参数: 可以是文本也可以是字典
        :param cookie: 可以是文本也可以是字典
        :param 协议头: 可以是文本也可以是字典
        :param 允许重定向: True 或 False 默认允许
        :param 代理地址: 账号:密码@IP:端口  或  IP:端口
        :param 编码: utf8,gbk·······
        :param 证书验证: 默认为False,需要引用证书时传入证书路径
        :param 上传文件: {'upload': ('code.png', 图片字节集, 'image/png')}
        :param 补全协议头: 默认补全常规协议头
        :param json: post提交参数时可能使用的类型
        :param 连接超时: 默认15
        :param 读取超时: 默认15
        :return: 返回网页对象
        """

        网页 = 网页返回类型()
        try:
            url = url.strip(' ')
            url = url if url.startswith('http') else 'http://' + url
            _cookie = {}
            _协议头 = {}
            传入参数 = {}
            if url.find('/', 8) != -1:
                host = url[url.find('://') + 3:url.find('/', 8)]
            else:
                host = url[url.find('://') + 3:]

            if type(协议头) == str:
                协议头数组 = 协议头.split('\n')
                for x in 协议头数组:
                    名称 = x[0:x.find(':')].strip(' ')
                    值 = x[x.rfind(名称) + len(名称) + 1:len(x)].strip(' ')
                    if 名称 and 值:
                        _协议头[名称] = 值
            else:
                _协议头 = 协议头

            if 补全协议头:
                if not 'Host' in _协议头:
                    _协议头['Host'] = host
                if not 'Accept' in _协议头:
                    _协议头['Accept'] = '*/*'
                if not 'Content-Type' in _协议头:
                    _协议头['Content-Type'] = 'application/x-www-form-urlencoded'
                if not 'User-Agent' in _协议头:
                    _协议头['User-Agent'] = 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.152 Safari/537.36'
                if not 'Referer' in _协议头:
                    _协议头['Referer'] = url

            if type(cookie) == str:
                if cookie.find(";") == -1:
                    名称 = cookie[0:cookie.find("=")].strip(' ')
                    值 = cookie[cookie.rfind(名称) + len(名称) + 1:len(cookie)].strip(' ')
                    if 名称 and 值:
                        _cookie = {名称: 值}
                else:
                    cookie数组 = cookie.split(';')
                    for x in cookie数组:
                        名称 = x[0:x.find("=")].strip(' ')
                        值 = x[cookie.rfind(名称) + len(名称) + 1:len(x)].strip(' ')
                        if 名称 and 值:
                            _cookie[名称] = 值
            else:
                _cookie = cookie

            传入参数['url'] = url
            传入参数['verify'] = 证书验证
            传入参数['cookies'] = _cookie
            传入参数['headers'] = _协议头
            传入参数['allow_redirects'] = 允许重定向
            if 参数:
                if 方式 == 0:
                    传入参数['params'] = 参数
                else:
                    传入参数['data'] = 参数
            if json:
                传入参数['json'] = json
            if 上传文件:
                传入参数['files'] = 上传文件
            if 代理地址:
                传入参数['proxies'] = {"http": "http://" + 代理地址, "https": "https://" + 代理地址}
            if 连接超时 and 读取超时:
                传入参数['timeout'] = (连接超时, 读取超时)

            # 发送
            if 方式 == 0:
                网页对象 = requests.get(**传入参数)
            elif 方式 == 1:
                网页对象 = requests.post(**传入参数)
            elif 方式 == 2:
                网页对象 = requests.put(**传入参数)
            elif 方式 == 3:
                网页对象 = requests.delete(**传入参数)
            elif 方式 == 4:
                网页对象 = requests.head(**传入参数)
            elif 方式 == 5:
                网页对象 = requests.options(**传入参数)

            if 编码:
                网页对象.encoding = 编码

            网页.原对象 = 网页对象
            网页.源码 = 网页对象.text
            网页.cookie = dict(网页对象.cookies)
            网页.状态码 = 网页对象.status_code
            网页.协议头 = 网页对象.headers
            网页.字节集 = 网页对象.content
            try:
                网页.json = 网页对象.json()
            except:
                pass
        except:
            print(sys._getframe().f_code.co_name, "函数发生异常", url)
            # print("错误发生时间：", str(datetime.datetime.now()))
            # print("错误的详细情况：", traceback.format_exc())
        return 网页


# @异常处理返回类型逻辑型
def 网页_访问(url, 方式=0, 参数='', cookie='', 协议头={}, 允许重定向=True, 代理地址=None, 编码=None,证书验证=False, 上传文件=None,补全协议头=True,json={}, 连接超时=15, 读取超时=15):
    """
    :param url: 链接,能自动补全htpp,去除首尾空格
    :param 方式: 0.get 1.post 2.put 3.delete 4.head 5.options
    :param 参数: 可以是文本也可以是字典
    :param cookie: 可以是文本也可以是字典
    :param 协议头: 可以是文本也可以是字典
    :param 允许重定向: True 或 False 默认允许
    :param 代理地址: 账号:密码@IP:端口  或  IP:端口
    :param 编码: utf8,gbk·······
    :param 证书验证: 默认为False,需要引用证书时传入证书路径
    :param 上传文件: {'upload': ('code.png', 图片字节集, 'image/png')}
    :param 补全协议头: 默认补全常规协议头
    :param json: post提交参数时可能使用的类型
    :param 连接超时: 默认15
    :param 读取超时: 默认15
    :return: 返回网页对象
    """

    网页 = 网页返回类型()
    url = url.strip(' ')
    url = url if url.startswith('http') else 'http://' + url
    _cookie = {}
    _协议头 = {}
    传入参数 = {}
    if url.find('/',8) != -1:
        host = url[url.find('://')+3:url.find('/', 8)]
    else:
        host = url[url.find('://')+3:]

    if type(协议头) == str:
        协议头数组 = 协议头.split('\n')
        for x in 协议头数组:
            名称 = x[0:x.find(':')].strip(' ')
            值 = x[x.rfind(名称) + len(名称)+1:len(x)].strip(' ')
            if 名称 and 值:
                _协议头[名称] = 值
    else:
        _协议头 = 协议头

    if 补全协议头:
        if not 'Host' in _协议头:
            _协议头['Host'] = host
        if not 'Accept' in _协议头:
            _协议头['Accept'] = '*/*'
        if not 'Content-Type' in _协议头:
            _协议头['Content-Type'] = 'application/x-www-form-urlencoded'
        if not 'User-Agent' in _协议头:
            _协议头['User-Agent'] = 'Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/42.0.2311.152 Safari/537.36'
        if not 'Referer' in _协议头:
            _协议头['Referer'] = url

    if type(cookie) == str:
        if cookie.find(";") == -1:
            名称 = cookie[0:cookie.find("=")].strip(' ')
            值 = cookie[cookie.rfind(名称) + len(名称) + 1:len(cookie)].strip(' ')
            if 名称 and 值:
                _cookie = {名称: 值}
        else:
            cookie数组 = cookie.split(';')
            for x in cookie数组:
                名称 = x[0:x.find("=")].strip(' ')
                值 = x[cookie.rfind(名称) + len(名称) + 1:len(x)].strip(' ')
                if 名称 and 值:
                    _cookie[名称] = 值
    else:
        _cookie = cookie


    传入参数['url'] = url
    传入参数['verify'] = 证书验证
    传入参数['cookies'] = _cookie
    传入参数['headers'] = _协议头
    传入参数['allow_redirects'] = 允许重定向
    if 参数:
        if 方式 == 0:
            传入参数['params'] = 参数
        else:
            传入参数['data'] = 参数
    if json:
        传入参数['json'] = json
    if 上传文件:
        传入参数['files'] = 上传文件
    if 代理地址:
        传入参数['proxies'] = {"http": "http://" + 代理地址, "https": "https://" + 代理地址}
    if 连接超时 and 读取超时:
        传入参数['timeout'] = (连接超时,读取超时)

    #发送
    if 方式 == 0:
        网页对象 = requests.get(**传入参数)
    elif 方式 == 1:
        网页对象 = requests.post(**传入参数)
    elif 方式 == 2:
        网页对象 = requests.put(**传入参数)
    elif 方式 == 3:
        网页对象 = requests.delete(**传入参数)
    elif 方式 == 4:
        网页对象 = requests.head(**传入参数)
    elif 方式 == 5:
        网页对象 = requests.options(**传入参数)


    if 编码:
        网页对象.encoding = 编码

    网页.原对象 = 网页对象
    网页.源码 = 网页对象.text
    try:
        网页.cookie = dict(网页对象.cookies)
    except:
        pass
    网页.状态码 = 网页对象.status_code
    网页.协议头 = 网页对象.headers
    网页.字节集 = 网页对象.content
    try:
        网页.json = 网页对象.json()
    except:
        pass
    # except:
        # print(sys._getframe().f_code.co_name, "函数发生异常",url)
        # print("错误发生时间：", str(datetime.datetime.now()))
        # print("错误的详细情况：", traceback.format_exc())
    return 网页



"""
兼容原生wx模块直接插上易函数的翅膀

"""
from .component.__init__ import *


class AnimationCtrl( 动画框 , 公用方法):
    pass


class BitmapButton( 图形按钮 , 公用方法):
    pass


class Button( 按钮 , 公用方法):
    pass


class CalendarCtrl(日历框  , 公用方法):
    pass


class CheckBox( 复选框 , 公用方法):
    pass


class CheckListBox( 选择列表框 , 公用方法):
    pass


class ColourPickerCtrl( 颜色选择器 , 公用方法):
    pass


class ComboBox( 组合框 , 公用方法):
    pass


class CommandLinkButton( 引导按钮 , 公用方法):
    pass


class DatePickerCtrl( 日期框 , 公用方法):
    pass


class EditableListBox( 排序列表框 , 公用方法):
    pass


class FloatSpin(  小数微调框, 公用方法):
    pass


class Frame( 窗口 , 公用方法):
    pass


class Gauge( 进度条 , 公用方法):
    pass


class GradientButton( 图文按钮L , 公用方法):
    pass


class HyperlinkCtrl( 超级链接框 , 公用方法):
    pass


class ListBox( 列表框 , 公用方法):
    pass


class ListCtrl( 超级列表框 , 公用方法):
    pass


class RadioBox( 分组单选框 , 公用方法):
    pass


class RadioButton( 单选框 , 公用方法):
    pass


class ScrollBar( 滚动条 , 公用方法):
    pass


class Slider( 滑块条 , 公用方法):
    pass


class SpinCtrl( 整数微调框 , 公用方法):
    pass


class StaticBitmap( 图片框 , 公用方法):
    pass


class StaticText( 标签 , 公用方法):
    pass


class TextCtrl( 编辑框 , 公用方法):
    pass


class ThemedGenBitmapTextButton( 图文按钮 , 公用方法):
    pass


class TimePickerCtrl( 时间框 , 公用方法):
    pass


class Panel( 容器 , 公用方法):
    pass

class Timer( 时钟 , 公用方法):
    pass


class Image( 图片操作 , 公用方法):
    pass




import wx
from .wxControl import *


class 编辑框(wx.TextCtrl, 公用方法):
    pass

    @property
    def 内容(self):
        return self.取内容()
    @内容.setter
    def 内容(self, value):
        self.置内容(value)



    def 置内容(self, 内容):
        self.SetValue(内容)

    def 取新文本样式(self):
        '返回当前用于新文本的样式。'
        return self.GetDefaultStyle()

    @组件_异常检测
    def 取指定行长度(self, 行号):
        '获取指定行的长度，不包括任何尾随换行符。'
        return self.GetLineLength(行号)

    @组件_异常检测
    def 取指定行内容(self, 行号):
        '返回文本控件中给定行的内容，不包括任何结尾的换行符。'
        return self.GetLineText(行号)

    def 取缓冲区行数(self):
        return self.GetNumberOfLines()

    def 内容是否被修改(self):
        '返回True文本是否已被用户修改。调用SetValue 不会使控件修改。'
        return self.IsModified()

    def 是否为多行编辑框(self):
        return self.IsMultiLine()

    def 是否为单行编辑框(self):
        return self.IsSingleLine()

    @组件_异常检测
    def 载入指定文件内容(self, 路径):
        '从指定文件加载内容到编辑框'
        return self.LoadFile(路径)

    @组件_异常检测
    def 内容写到指定文件(self, 路径):
        '将编辑框的内容写到指定文件内'
        return self.SaveFile(路径)

    def 标记为已修改2(self):
        '将文本标记为已修改'
        return self.MarkDirty()

    @组件_异常检测
    def 指定位置转像素位置(self, 位置):
        '取指定位置处的文本的像素坐标'
        return self.PositionToCoords(位置)

    @组件_异常检测
    def 指定位置转行列位置(self, 位置):
        '取指定位置处的文本所在行跟列,返回一个元组,(是否存在,行,列)'
        return self.PositionToXY(位置)

    @组件_异常检测
    def 置新文本样式(self, 样式):
        '更改要用于要添加到控件的新文本的默认样式。'
        return self.SetDefaultStyle(样式)

    @组件_异常检测
    def 置修改状态(self, 修改=True):
        '将控件标记为是否被用户修改'
        return self.SetModified(修改)

    def 标记为已修改(self):
        '将控件标记为已被用户修改'
        return self.SetModified(True)

    def 标记为未修改(self):
        '将控件标记为未被用户修改'
        return self.SetModified(False)

    @组件_异常检测
    def 置指定范围样式(self, 开始位置, 结束位置, 样式):
        return self.SetStyle(开始位置, 结束位置, 样式)

    @组件_异常检测
    def 置指定位置可见(self, 位置):
        '使指定位置的字符显示在编辑框可见范围内'
        return self.ShowPosition(位置)

    @组件_异常检测
    def 指定行列转位置(self, 行, 列):
        '将给定的从零开始的列和行号转换为位置'
        return self.XYToPosition(行, 列)

    @组件_异常检测
    def 加入文本(self, 内容):
        return self.write(内容)

    def 清空内容(self):
        self.Clear()



import re



class 正则:
    """
    正则的匹配模式

    re.A
    re.ASCII
    让 \w, \W, \b, \B, \d, \D, \s 和 \S 只匹配ASCII，而不是Unicode。这只对Unicode样式有效，会被byte样式忽略。相当于前面语法中的内联标志 (?a) 。

    注意，为了保持向后兼容， re.U 标记依然存在（还有他的同义 re.UNICODE 和嵌入形式 (?u) ) ， 但是这些在 Python 3 是冗余的，因为默认字符串已经是Unicode了（并且Unicode匹配不允许byte出现)。

    re.DEBUG
    显示编译时的debug信息，没有内联标记。

    re.I
    re.IGNORECASE
    进行忽略大小写匹配；表达式如 [A-Z] 也会匹配小写字符。Unicode匹配（比如 Ü 匹配 ü）同样有用，除非设置了 re.ASCII 标记来禁用非ASCII匹配。当前语言区域不会改变这个标记，除非设置了 re.LOCALE 标记。这个相当于内联标记 (?i) 。

    注意，当设置了 IGNORECASE 标记，搜索Unicode样式 [a-z] 或 [A-Z] 的结合时，它将会匹配52个ASCII字符和4个额外的非ASCII字符： 'İ' (U+0130, 拉丁大写的 I 带个点在上面), 'ı' (U+0131, 拉丁小写没有点的 I ), 'ſ' (U+017F, 拉丁小写长 s) and 'K' (U+212A, 开尔文符号).如果使用 ASCII 标记，就只匹配 'a' 到 'z' 和 'A' 到 'Z' 。

    re.L
    re.LOCALE
    由当前语言区域决定 \w, \W, \b, \B 和大小写敏感匹配。这个标记只能对byte样式有效。这个标记不推荐使用，因为语言区域机制很不可靠，它一次只能处理一个 "习惯”，而且只对8位字节有效。Unicode匹配在Python 3 里默认启用，并可以处理不同语言。 这个对应内联标记 (?L) 。

    在 3.6 版更改: re.LOCALE 只能用于byte样式，而且不能和 re.ASCII 一起用。

    在 3.7 版更改: 设置了 re.LOCALE 标记的编译正则对象不再在编译时依赖语言区域设置。语言区域设置只在匹配的时候影响其结果。

    re.M
    re.MULTILINE
    设置以后，样式字符 '^' 匹配字符串的开始，和每一行的开始（换行符后面紧跟的符号）；样式字符 '$' 匹配字符串尾，和每一行的结尾（换行符前面那个符号）。默认情况下，’^’ 匹配字符串头，'$' 匹配字符串尾。对应内联标记 (?m) 。

    re.S
    re.DOTALL
    让 '.' 特殊字符匹配任何字符，包括换行符；如果没有这个标记，'.' 就匹配 除了 换行符的其他任意字符。对应内联标记 (?s) 。

    re.X
    re.VERBOSE
    这个标记允许你编写更具可读性更友好的正则表达式。通过分段和添加注释。空白符号会被忽略，除非在一个字符集合当中或者由反斜杠转义，或者在 *?, (?: or (?P<…> 分组之内。当一个行内有 # 不在字符集和转义序列，那么它之后的所有字符都是注释。

    例如正则表达式 \d +  # 这是注释

    """
    只匹配ASCII = re.A
    忽略大小写 = re.I
    语言依赖 = re.L
    多行模式 = re.M
    点dot匹配全部字符 = re.S
    只匹配Unicode = re.U
    允许注释 = re.X


class 正则表达式():
    """
    python的正则表达式
    https://docs.python.org/zh-cn/3/library/re.html#re.compile

    例子

        print(正则表达式(r"\d+", 正则.多行模式 | 正则.忽略大小写).搜索("123456789"))
        print(正则表达式(r"\d+", 正则.多行模式 | 正则.忽略大小写).替换("替换", "123456798"))

    """

    def __init__(self, 正则表达式文本: str = "", 模式=正则.忽略大小写 | 正则.多行模式):
        self.创建(正则表达式文本, 模式)

    def 创建(self, 正则表达式文本, 模式=正则.忽略大小写 | 正则.多行模式):
        """
        模式 正则.多行模式 | 正则.忽略大小写
            正则.只匹配ASCII = re.A
            正则.忽略大小写 = re.I
            正则.语言依赖 = re.L
            正则.多行模式 = re.M
            正则.点dot匹配全部字符 = re.S
            正则.只匹配Unicode = re.U
            正则.允许注释 = re.X

        :param 正则表达式文本:
        :param 模式:
        :return:

        """
        self.prog = re.compile(正则表达式文本, flags=模式)
        return self

    def 搜索(self, 内容):
        self.res = self.prog.findall(内容)
        return self.res

    def 替换(self, 用作替换的文本: str, 内容: str = "") -> str:
        return re.sub(self.prog, 用作替换的文本, 内容)



class 正则表达式类():
    """
    精易模块的正则表达式类命名方式

    坦率讲 该类是为了精易模块的正则表达式代码能够直接迁移

    python有更好用的方式实现



    传统易语言代码的实现 取子匹配数量 取子匹配文本 在python里面很多余 为了兼容实现了
        zz = 正则表达式类()
        zz.创建(r"\d+", str, 是否区分大小写=True, 是否匹配多行=True)
        for i in range(zz.取匹配数量()):
            print("取匹配文本", zz.取匹配文本(i))
            print("取子匹配数量", zz.取子匹配数量())
            for k in range(zz.取子匹配数量()):
                print("取子匹配文本 i= %s k= %s" % (i, k), zz.取子匹配文本(i, k))
    python的实现
        正则结果 = 正则表达式类(r"\d+", str, 是否区分大小写=True, 是否匹配多行=True).取结果()


    建议使用 正则表达式 代码更加简单 本类 只是为了兼容精易模块的代码降低学习成本

    print(正则表达式(r"\d+", 正则.多行模式 | 正则.忽略大小写).搜索("123456789"))
    print(正则表达式(r"\d+", 正则.多行模式 | 正则.忽略大小写).替换("替换", "123456798"))

    """

    def __init__(self, 正则表达式文本: str = "", 被搜索的文本: str = "", 是否区分大小写=False, 是否匹配多行=True):
        pass
        self.创建(正则表达式文本, 被搜索的文本, 是否区分大小写, 是否匹配多行)

    def 创建(self, 正则表达式文本: str, 被搜索的文本: str = "", 是否区分大小写=False, 是否匹配多行=True):
        pass
        if 正则表达式文本 == "":
            return self

        self.模式 = 0
        if 是否区分大小写 == False:
            self.模式 = self.模式 | re.I
        if 是否匹配多行:
            self.模式 = self.模式 | re.M
        # if 是否全局匹配: python没

        # print(正则表达式文本)
        # print(被搜索的文本)
        self.正则表达式文本 = 正则表达式文本
        self.prog = re.compile(正则表达式文本, flags=self.模式)

        self.匹配(被搜索的文本)

        return self

    def 匹配(self, 被搜索的文本: str, 模式=0):
        """
        模式
            re.A(只匹配ASCII字符),
            re.I(忽略大小写),
            re.L(语言依赖),
            re.M(多行模式),
            re.S(点dot匹配全部字符),
            re.U(Unicode匹配),
            and re.X(冗长模式)。

        :param 被搜索的文本:
        :param 模式: re.A|re.I
        :return:
        """
        self.被搜索的文本 = 被搜索的文本
        if (模式 == 0):
            self.res = self.prog.findall(self.被搜索的文本)
        else:
            self.res = self.prog.findall(self.被搜索的文本, 模式)

        # print(self.res)
        return self

    def 取匹配数量(self) -> int:
        pass

        return len(self.res)

    def 取匹配文本(self, 匹配索引: int) -> str:
        pass
        try:
            col = self.res[匹配索引]
            return col
        except:
            pass
        return False

    def 取子匹配文本(self, 匹配索引: int, 子表达式索引: int) -> str:
        """

        :param 匹配索引:
        :param 子表达式索引:
        :return:
        """
        # pass
        try:
            col = self.res[匹配索引]
            if (len(col) <= 子表达式索引):
                return False
            return col[子表达式索引]
        except:
            pass
        return False

    def 取子匹配数量(self) -> int:
        # print(self.res[0])
        # print(len(self.res[0]))
        try:
            return len(self.res[0])
        except:
            pass
        return 0

    def 取结果(self) -> str:
        return self.res

    def 替换(self, 用作替换的文本: str, 被搜索的文本: str = "") -> str:
        if (被搜索的文本 != ""):
            self.被搜索的文本 = 被搜索的文本
        return re.sub(self.prog, 用作替换的文本, self.被搜索的文本)




from .stringBase import *
import re
import random


def strCut(内容: str, 表达式: str) -> str:
    subbds = 表达式.split("$")
    if (len(subbds) == 2):
        return 文本_取出中间文本(内容, subbds[0], subbds[1])

    if (len(subbds) == 1):
        return 文本_取出中间文本(内容, subbds[0], "")
    return 内容


def 文本_取左边(需取文本: str, 欲寻找的文本: str) -> str:
    return 文本_取出中间文本(需取文本, "", 欲寻找的文本)


def 文本_取右边(需取文本: str, 欲寻找的文本: str) -> str:
    return 文本_取出中间文本(需取文本, 欲寻找的文本, "")


def 文本_取出中间文本(需取文本: str, 左边文本: str, 右边文本: str) -> str:
    leftLen = len(左边文本)
    leftP = 需取文本.index(左边文本)
    if (leftP == -1):
        return ''
    if (右边文本 == ""):
        rigthP = len(需取文本) + 1
        leftP = leftP + leftLen
    else:
        leftP = leftP + leftLen
        rigthP = 需取文本.index(右边文本, leftP)
        if (rigthP == - 1):
            return ''
    return 需取文本[leftP: rigthP]


def 文本_取随机字母(取出的数量: int, 类型: int = 2) -> str:
    '类型：0.小写 1.大写 2.混合'
    取出的数量 = 1 if 取出的数量 < 1 else 取出的数量
    类型 = 0 if 类型 < 0 or 类型 > 2 else 类型
    字母 = 'abcdefghijklnmopqrstuvwxyz'
    文本 = ''
    if 类型 == 0:
        for x in range(取出的数量):
            文本 += random.choice(字母)
    elif 类型 == 1:
        for x in range(取出的数量):
            文本 += random.choice(字母).upper()
    else:
        for x in range(取出的数量):
            随机数 = random.randint(0, 1)
            if 随机数 == 1:
                文本 += random.choice(字母)
            else:
                文本 += random.choice(字母).upper()
    return 文本


def 文本_取随机字母和数字(取出的数量: int) -> str:
    '包括0-9 a-z A-Z'
    取出的数量 = 1 if 取出的数量 < 1 else 取出的数量
    字符 = '0123456789abcdefghijklnmopqrstuvwxyzABCDEFGHIJKLNMOPQRSTUVWXYZ'
    文本 = ''
    for x in range(取出的数量):
        文本 += random.choice(字符)
    return 文本


def 文本_取随机数字(取出的数量: int) -> str:
    取出的数量 = 1 if 取出的数量 < 1 else 取出的数量
    字符 = '0123456789'
    文本 = ''
    for x in range(取出的数量):
        文本 += random.choice(字符)
    return 文本


def 文本_取随机汉字(取出的数量: int) -> str:
    '部分常见汉字'
    取出的数量 = 1 if 取出的数量 < 1 else 取出的数量
    文本 = ''
    部分汉字 = "的一是了我不人在他有这个上们来到时大地为子中你说生国年着就那和要她出也得里后自以会家可下而过天去能对小多然于心学么之都好看起发当没成只如事把还用第样道想作种开美总从无情己面最女但现前些所同日手又行意动方期它头经长儿回位分爱老因很给名法间斯知世什两次使身者被高已亲其进此话常与活正感见明问力理尔点文几定本公特做外孩相西果走将月十实向声车全信重三机工物气每并别真打太新比才便夫再书部水像眼等体却加电主界门利海受听表德少克代员许稜先口由死安写性马光白或住难望教命花结乐色更拉东神记处让母父应直字场平报友关放至张认接告入笑内英军候民岁往何度山觉路带万男边风解叫任金快原吃妈变通师立象数四失满战远格士音轻目条呢病始达深完今提求清王化空业思切怎非找片罗钱紶吗语元喜曾离飞科言干流欢约各即指合反题必该论交终林请医晚制球决窢传画保读运及则房早院量苦火布品近坐产答星精视五连司巴奇管类未朋且婚台夜青北队久乎越观落尽形影红爸百令周吧识步希亚术留市半热送兴造谈容极随演收首根讲整式取照办强石古华諣拿计您装似足双妻尼转诉米称丽客南领节衣站黑刻统断福城故历惊脸选包紧争另建维绝树系伤示愿持千史谁准联妇纪基买志静阿诗独复痛消社算义竟确酒需单治卡幸兰念举仅钟怕共毛句息功官待究跟穿室易游程号居考突皮哪费倒价图具刚脑永歌响商礼细专黄块脚味灵改据般破引食仍存众注笔甚某沉血备习校默务土微娘须试怀料调广蜖苏显赛查密议底列富梦错座参八除跑亮假印设线温虽掉京初养香停际致阳纸李纳验助激够严证帝饭忘趣支春集丈木研班普导顿睡展跳获艺六波察群皇段急庭创区奥器谢弟店否害草排背止组州朝封睛板角况曲馆育忙质河续哥呼若推境遇雨标姐充围案伦护冷警贝著雪索剧啊船险烟依斗值帮汉慢佛肯闻唱沙局伯族低玩资屋击速顾泪洲团圣旁堂兵七露园牛哭旅街劳型烈姑陈莫鱼异抱宝权鲁简态级票怪寻杀律胜份汽右洋范床舞秘午登楼贵吸责例追较职属渐左录丝牙党继托赶章智冲叶胡吉卖坚喝肉遗救修松临藏担戏善卫药悲敢靠伊村戴词森耳差短祖云规窗散迷油旧适乡架恩投弹铁博雷府压超负勒杂醒洗采毫嘴毕九冰既状乱景席珍童顶派素脱农疑练野按犯拍征坏骨余承置臓彩灯巨琴免环姆暗换技翻束增忍餐洛塞缺忆判欧层付阵玛批岛项狗休懂武革良恶恋委拥娜妙探呀营退摇弄桌熟诺宣银势奖宫忽套康供优课鸟喊降夏困刘罪亡鞋健模败伴守挥鲜财孤枪禁恐伙杰迹妹藸遍盖副坦牌江顺秋萨菜划授归浪听凡预奶雄升碃编典袋莱含盛济蒙棋端腿招释介烧误乾坤"
    for x in range(取出的数量):
        文本 += random.choice(部分汉字)
    return 文本


def 文本_取随机姓氏(取常见姓氏=False) -> str:
    '常见姓氏为自己设置挑选的,仅做参考'
    百家姓 = """赵钱孙李周吴郑王冯陈褚卫蒋沈韩杨朱秦尤许何吕施张孔曹严华金魏陶姜戚谢邹喻柏水窦章云苏潘葛奚范彭郎鲁韦昌马苗凤花方俞任袁柳酆鲍史唐费廉岑薛雷贺倪汤滕殷罗毕郝邬安常乐于时傅皮卞齐康伍余元卜顾孟平黄和穆萧尹姚邵湛汪祁毛禹狄米贝明臧计伏成戴谈宋茅庞熊纪舒屈项祝董梁杜阮蓝闵席季麻强贾路娄危江童颜郭梅盛林刁钟徐邱骆高夏蔡田樊胡凌霍虞万支柯昝管卢莫柯房裘缪干解应宗丁宣贲邓郁单杭洪包诸左石崔吉钮龚程嵇邢滑裴陆荣翁荀羊于惠甄曲家封芮羿储靳汲邴糜松井段富巫乌焦巴弓牧隗山谷车侯宓蓬全郗班仰秋仲伊宫宁仇栾暴甘钭历戎祖武符刘景詹束龙叶幸司韶郜黎蓟溥印宿白怀蒲邰从鄂索咸籍赖卓蔺屠蒙池乔阳郁胥能苍双闻莘党翟谭贡劳逄姬申扶堵冉宰郦雍却璩桑桂濮牛寿通边扈燕冀浦尚农温别庄晏柴瞿阎充慕连茹习宦艾鱼容向古易慎戈廖庾终暨居衡步都耿满弘匡国文寇广禄阙东欧殳沃利蔚越夔隆师巩厍聂晁勾敖融冷訾辛阚那简饶空曾毋沙乜养鞠须丰巢关蒯相查后荆红游竺权逮盍益桓公"""
    常见百家姓 = '赵钱孙李周吴郑王冯陈褚卫蒋沈韩杨朱秦尤许何吕施张孔曹严华金魏陶姜戚谢邹喻柏水窦章云苏潘葛范彭郎鲁韦昌马苗凤花方俞任袁柳史唐费薛雷贺罗毕于齐萧尹姚顾孟平黄宋庞项祝董梁杜阮刘万丁石洪白田夏'
    if 取常见姓氏 == False:
        return random.choice(百家姓)
    else:
        return random.choice(常见百家姓)


def 文本_取随机手机号() -> str:
    号码前缀 = ['130', '131', '132', '134', '135', '136', '137', '138', '139', '147', '150', '151', '152', '153', '155',
            '156', '157', '158', '159', '170', '171', '180', '182', '183', '185', '186', '187', '188', '189']
    尾号 = ''
    for x in range(8):
        尾号 += str(random.randint(0, 9))
    return random.choice(号码前缀) + 尾号


def 文本_取随机邮箱() -> str:
    邮箱后缀 = ['@qq.com', '@sina.com', '@126.com', '@163.com', '@hotmail.com', '@139.com', '@189.com', '@sohu.com',
            '@21cn.com', '@189.com', '@tom.com', '@aol.com', '@263.com', '@aliyun.com', '@foxmail.com', '@yeah.net']
    字母 = 字符 = 混合 = ""
    数字 = str(random.randint(1, 9))
    for x in range(random.randint(5, 11)): 数字 += str(random.randint(0, 9))
    for x in range(random.randint(5, 11)): 字母 += random.choice('qwertyuiopasdfghjklzxcvbnm')
    for x in range(random.randint(6, 11)): 字符 += random.choice('0123456789abcdefghijklnmopqrstuvwxyz')
    for x in range(random.randint(1, 6)): 混合 += random.choice('qwertyuiopasdfghjklzxcvbnm')
    for x in range(random.randint(3, 10)): 混合 += str(random.randint(0, 9))
    字符 = 字符[1:] if 字符.startswith('0') else 字符
    return random.choice([混合, 混合, 混合, 字母, 数字, 字符]) + random.choice(邮箱后缀)


def 文本_删左边(欲处理文本: str, 删除长度: int) -> str:
    return 取文本右边(欲处理文本, 取文本长度(欲处理文本) - 删除长度)


def 文本_删右边(欲处理文本: str, 删除长度: int) -> str:
    return 取文本左边(欲处理文本, 取文本长度(欲处理文本) - 删除长度)


def 文本_删中间(欲处理文本: str, 起始位置: int, 删除长度: int) -> str:
    return 取文本左边(欲处理文本, 起始位置) + 文本_删左边(欲处理文本, 起始位置 + 删除长度)


def 文本_取出文本中汉字(欲处理文本: str) -> str:
    '返回文本中的所有汉字'
    result = re.findall(r"[\u4e00-\u9fa5]", 欲处理文本)
    return "".join(result)


def 文本_逐字分割(欲处理文本: str) -> str:
    arr = []
    for item in 欲处理文本:
        arr.append(item)
    return arr


def 文本_颠倒(欲处理文本: str) -> str:
    return 欲处理文本[::-1]


def 文本_是否为汉字(欲处理文本: str) -> bool:
    '全汉字返回True'
    zhmodel = re.compile(u'[^\u4e00-\u9fa5]')  # 检查中文
    match = zhmodel.search(欲处理文本)
    if match:
        return False
    else:
        return True


# type 0大小写字母 1大写字母 2小写字母
def 文本区分_只取字母(欲处理文本: str, type=0) -> str:
    '返回文本中的所有小写字母'
    if type == 0:
        strArr = re.findall(r'[a-zA-Z]+', 欲处理文本)
    elif type == 1:
        strArr = re.findall(r'[A-Z]+', 欲处理文本)
    elif type == 2:
        strArr = re.findall(r'[a-z]+', 欲处理文本)
    else:
        pass
    if strArr:
        return "".join(strArr)


def 文本区分_只取数字(欲处理文本: str) -> str:
    '返回文本中的所有数字'
    strArr = re.findall(r'\d+', 欲处理文本)
    if strArr:
        return "".join(strArr)


def 判断文本(文本: str, 关键字: list) -> bool:
    for item in 关键字:
        index = 寻找文本(文本, item)
        if index != -1:
            return True
    return False


def 判断文本s(文本: str, 关键字: list) -> bool:
    for item in 关键字:
        index = 寻找文本(文本, item)
        if index != -1:
            return item
    return ""


def 文本_取手机号码(欲处理文本: str) -> str:
    '取文本中的手机号码,列表格式'
    result = re.findall(r"1[3,4,5,6,7,8,9]\d{9}", 欲处理文本)
    if result:
        return result


def 文本_取IP地址(欲处理文本: str) -> str:
    '取文本中的IP地址,列表格式'
    result = re.findall(r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}", 欲处理文本)
    if result:
        return result


def 文本_取电话号码(欲处理文本: str) -> str:
    '取文本中的电话号码,列表格式'
    result = re.findall(r"\d{3,4}[\s,-]?\d{7,8}", 欲处理文本)
    if result:
        return result


def 文本_取QQ号码(欲处理文本: str) -> str:
    '取文本中的QQ号码,列表格式,最多取11位数QQ号'
    result = re.findall(r"[1-9][0-9]{4,10}", 欲处理文本)
    if result:
        return result


def 文本_取邮政编码(欲处理文本: str) -> str:
    '中国邮政编码为6位数字,列表格式'
    result = re.findall(r"[1-9]\d{5}(?!\d)", 欲处理文本)
    if result:
        return result


def 文本_取身份证号码(欲处理文本: str) -> str:
    '中国的身份证为15位或18位,列表格式'
    result = re.findall(r"[1-9][0-9,X]{14,17}", 欲处理文本)
    if result:
        return result


def 文本_取双字节字符(欲处理文本: str) -> str:
    '汉字 大写符号是双字节 数字 字母 小写符号是单字节字符'
    result = re.findall(r"[^\x00-\xff]", 欲处理文本)
    return "".join(result)


def 文本_取网址(欲处理文本: str) -> str:
    '取带：//的网址,列表格式'
    result = re.findall(r"[a-zA-z]+://[^\s]*", 欲处理文本)
    if result:
        return result


def 文本_取IP跟端口(欲处理文本: str) -> str:
    '取IP地址带端口号,如：127.0.0.1:8080'
    result = re.findall(r"\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}:\d{2,5}", 欲处理文本)
    if result:
        return result


def 文本_取邮箱号码(欲处理文本: str) -> str:
    '取邮箱号码,列表格式'
    result = re.findall(r"[a-z0-9\.\-+_]{1,30}@[a-z0-9\.\-+_]{1,30}\.[a-z]{1,10}", 欲处理文本)
    if result:
        return result


def 文本_大小写翻转(欲处理文本: str) -> str:
    return 欲处理文本.swapcase()


def 文本_是否为大写字母(欲处理文本: str) -> bool:
    return 欲处理文本.isupper()


def 文本_是否为小写字母(欲处理文本: str) -> bool:
    return 欲处理文本.islower()


def 文本_是否为字母(欲处理文本: str) -> bool:
    return 欲处理文本.isalpha()


def 文本_是否为数字字母(欲处理文本: str) -> bool:
    return 欲处理文本.isalnum()


def 文本_是否为数字(欲处理文本: str) -> bool:
    return 欲处理文本.isdigit()


def 文本_取出现次数(欲处理文本: str, 欲查询的文本, 开始的位置=0, 结束的位置=0) -> int:
    结束的位置 = len(欲处理文本) if 结束的位置 < 1 else 结束的位置
    return 欲处理文本.count(欲查询的文本, 开始的位置, 结束的位置)


def 文本_单词首字母大写(欲处理文本) -> str:
    return 欲处理文本.title()


def 文本_填充空格_居中(欲处理文本: str, 填充目标长度: int) -> str:
    '将文本用空格填充到指定长度使欲处理文本居中'
    return 欲处理文本.center(填充目标长度)


def 文本_填充空格_左对齐(欲处理文本: str, 填充目标长度: int) -> str:
    '将文本用空格填充到指定长度使欲处理文本左对齐'
    return 欲处理文本.ljust(填充目标长度)


def 文本_填充空格_右对齐(欲处理文本: str, 填充目标长度: int) -> str:
    '将文本用空格填充到指定长度使欲处理文本右对齐'
    return 欲处理文本.rjust(填充目标长度)


def 文本_自动补零(欲处理文本: str, 填充目标长度: int) -> str:
    '将文本用0填充到指定长度使欲处理文本右对齐,如：0001'
    return 欲处理文本.zfill(填充目标长度)


def 数组_转文本(数组: list, 分隔符: str = "\r\n") -> str:
    return 分隔符.join(数组)


def 文本_判断文本前缀(欲处理文本: str, 开头的文本: str, 开始的位置: int = 0, 结束的位置: int = 0) -> bool:
    '如果字符串为指定的开头返回True，否则返回False'
    结束的位置 = len(欲处理文本) if 结束的位置 < 1 else 结束的位置
    return 欲处理文本.startswith(开头的文本, 开始的位置, 结束的位置)


def 文本_判断文本后缀(欲处理文本: str, 结尾的文本: str, 开始的位置: int = 0, 结束的位置: int = 0) -> str:
    '如果字符串为指定的后缀返回True，否则返回False'
    结束的位置 = len(欲处理文本) if 结束的位置 < 1 else 结束的位置
    return 欲处理文本.endswith(结尾的文本, 开始的位置, 结束的位置)


def 文本_TAB转空格(欲处理文本: str, 转换的数量: int = 8) -> str:
    '把字符串中的 tab 符号（\t）转换为空格，如不指定参数，默认的空格数是 tabsize=8'
    return 欲处理文本.expandtabs(tabsize=转换的数量)


def 文本_取随机IP() -> str:
    return '.'.join(str(random.randint(0, 255)) for _ in range(4))


def 文本_取中间_批量(欲处理文本: str, 前面的文本: str, 后面的文本: str) -> str:
    index = 0
    list = []
    while True:
        index = 欲处理文本.find(前面的文本, index)
        if index != -1:
            lastIndex = 欲处理文本.find(后面的文本, index + len(前面的文本))
            if lastIndex != -1:
                index = index + len(前面的文本)
                str = 欲处理文本[index:lastIndex]
                if len(str) > 0:
                    list.append(str)
            else:
                break
        else:
            break
    return list


# 使用教程
# https://github.com/mozillazg/python-pinyin
def 文本_汉字转拼音(欲处理文本, 连接符='', 拼音风格=0, 遍历多音=False, 无拼音处理=0, 严格规范=False):
    from pypinyin import Style, pinyin  # 需要安装拼音库 pip install pypinyin
    '默认返回全拼,拼音风格:0是不带声调的全拼,1是带声调的全拼,2是取声母部分,3是取首字母,无拼音处理：0是保留原始字符,1是忽略该字符,2是 替换为去掉 \\u 的 unicode 编码字符串'
    风格 = [Style.NORMAL, Style.TONE, Style.INITIALS, Style.FIRST_LETTER]
    处理 = ['default', 'ignore', 'replace']
    rStyle = 风格[0 if 拼音风格 > 3 or 拼音风格 < 0 else 拼音风格]
    Errors = 处理[0 if 无拼音处理 > 2 or 无拼音处理 < 0 else 无拼音处理]

    拼音 = ''
    拼音列表 = pinyin(hans=欲处理文本, style=rStyle, heteronym=遍历多音, errors=Errors, strict=严格规范)
    for x in 拼音列表:
        for i in x:
            拼音 = 拼音 + 连接符 + i
    return 拼音



"""
编码转换
"""

import binascii, hashlib, base64, hmac
from urllib import parse
from .public import *
import chardet
import io


def 编码_检查(bytes):
    """
    检测文件编码
    #Return:
    #    fileencoding: 文件编码
    #    confidence: 检测结果的置信度，百分比
    """
    detector = chardet.detect(bytes)
    # confidence = detector['confidence']
    if detector['encoding'] is None:
        detector['encoding'] = 'unknown'
        # confidence = 0.99
    return detector['encoding']


def 文本编码转换(内容: str, 来源编码: str, 目标编码: str):
    if 来源编码 == "":
        s = 内容
        来源编码 = 编码_检查(s)
        if 来源编码 == 目标编码:
            return 内容
        if not (来源编码 == "gbk" or 来源编码 == "utf-8" or 来源编码 == "GB2312"):
            return 内容

    with io.BytesIO(内容) as data:
        with io.TextIOWrapper(data, encoding=来源编码, errors='replace') as f:
            newtext = f.read().encode(encoding=目标编码)
    return newtext


@异常处理返回类型逻辑型
def 编码_UTF8编码(内容):
    return 内容.encode(encoding='UTF-8', errors='strict')


@异常处理返回类型逻辑型
def 编码_UTF8解码(内容):
    return 内容.decode(encoding='UTF-8', errors='strict')


@异常处理返回类型逻辑型
def 编码_URL编码(内容):
    return parse.quote(内容)


@异常处理返回类型逻辑型
def 编码_URL解码(内容):
    return parse.unquote(内容)


@异常处理返回类型逻辑型
def 编码_GBK编码(内容):
    return 内容.encode(encoding='GBK', errors='strict')


@异常处理返回类型逻辑型
def 编码_GBK解码(内容):
    return 内容.decode(encoding='GBK', errors='strict')


@异常处理返回类型逻辑型
def 编码_ANSI到USC2(内容):
    return ascii(内容)


@异常处理返回类型逻辑型
def 编码_USC2到ANSI(内容):
    return 内容.encode('utf-8').decode('unicode_escape')


@异常处理返回类型逻辑型
def 编码_BASE64编码(内容):
    '要编码的内容支持文本跟字节集'
    if type(内容) == str:
        return base64.b64encode(内容.encode('UTF-8')).decode("UTF-8")
    else:
        return base64.b64encode(内容).decode("UTF-8")


@异常处理返回类型逻辑型
def 编码_BASE64解码(内容, 返回字节集=False):
    '字节集用于解码图片'
    if 返回字节集:
        return base64.b64decode(内容)
    else:
        return base64.b64decode(内容).decode("UTF-8")


@异常处理返回类型逻辑型
def 加密_MD5(内容, 编码="utf-8"):
    MD5 = hashlib.md5()
    MD5.update(内容.encode(encoding=编码))
    return MD5.hexdigest()


@异常处理返回类型逻辑型
def 加密_SHA(内容, 方式=0):
    '方式： 0.SHA1 1.SHA224 2.SHA256 3.SHA384 4.SHA512'
    字典 = {0: hashlib.sha1(), 1: hashlib.sha224(), 2: hashlib.sha256(), 3: hashlib.sha384(), 4: hashlib.sha512()}
    字典[方式].update(str(内容).encode('utf-8'))
    return 字典[方式].hexdigest()


@异常处理返回类型逻辑型
def 加密_SHA3(内容, 方式=0):
    '方式： 0.SHA224 1.SHA256 2.SHA384 3.SHA512'
    字典 = {0: hashlib.sha3_224(), 1: hashlib.sha3_256(), 2: hashlib.sha3_384(), 3: hashlib.sha3_512()}
    字典[方式].update(str(内容).encode('utf-8'))
    return 字典[方式].hexdigest()


@异常处理返回类型逻辑型
def 加密_HmacSHA256(key, 内容):
    return base64.b64encode(
        hmac.new(bytes(key, encoding='utf-8'), bytes(内容, encoding='utf-8'), digestmod=hashlib.sha256).digest()).decode(
        "utf-8")


@异常处理返回类型逻辑型
def 加密_CRC32(内容):
    return binascii.crc32(内容.encode("utf-8"))


@异常处理返回类型逻辑型
def 进制_十到二(内容):
    return bin(内容)


@异常处理返回类型逻辑型
def 进制_十到八(内容):
    return oct(内容)


@异常处理返回类型逻辑型
def 进制_十到十六(内容):
    return hex(内容)


@异常处理返回类型逻辑型
def 进制_二到十(内容):
    return int(内容, base=2)


@异常处理返回类型逻辑型
def 进制_八到十(原内容):
    return int(原内容, base=8)


@异常处理返回类型逻辑型
def 进制_十六到十(内容):
    return int(内容, base=16)



import unittest

from .binary import *


class TestBinary(unittest.TestCase):

    def test_1(self):
        data = binary编码(b"123456798")
        self.assertEqual(data, b"313233343536373938")
        data = binary解码(data)
        self.assertEqual(data, b"123456798")

        data = binary编码("你好")
        self.assertEqual(data, b"e4bda0e5a5bd")
        data = binary解码(data)
        self.assertEqual(data, b'\xe4\xbd\xa0\xe5\xa5\xbd')
        self.assertEqual(data.decode("utf-8"), "你好")



"""
时钟

易语言的时钟组件

实现了 时钟周期事件

使用装饰器设置定时任务 函数内返回 false则停止时钟


@时钟周期事件(时钟周期=1000)
def 定时任务():
    print("定时任务")
    # return True
    return False

定时任务2运行状态 = True

def 定时任务2():
    print(1111111111111,定时任务2运行状态)
    return 定时任务2运行状态
    # return False


时钟(定时任务2, 时钟周期=2000)

@时钟周期事件(时钟周期=5000)
def 定时任务3():
    global 定时任务2运行状态
    print("把时钟停止掉")
    定时任务2运行状态 = False
    return False


"""

from threading import Timer


def 时钟(func, 时钟周期):
    """
    时钟 定时调用函数  也可以使用装饰器 @时钟周期事件(时钟周期=1000)

    def 定时任务():
        print("定时任务")
        return False 停止时钟周期事件

    时钟(定时任务, 时钟周期=2000)

    :param func: 运行函数 函数返回false 则停止时钟周期事件
    :param 时钟周期: 毫秒
    """
    时钟周期 = int(时钟周期 / 1000)
    def 自调用函数():
        if not func() == False:
            return Timer(时钟周期, 自调用函数).start()
    return Timer(时钟周期, 自调用函数).start()

def 时钟周期事件(*args, **kwargs):
    """
    装饰器

    @时钟周期事件(时钟周期=1000)
    def 定时任务():
        print("定时任务")
    """
    def warpp(func):
        时钟(func, kwargs['时钟周期'])
    return warpp


import re
import os
import time

# 设置时区
os.environ['TZ'] = 'Asia/Shanghai'

def get_version(package):
    """
    Return package version as listed in `__version__` in `__init__.py`.
    """
    init_py = open(os.path.join(package, '__init__.py')).read()
    mth = re.search("__version__\s?=\s?['\"]([^'\"]+)['\"]", init_py)
    if mth:
        return mth.group(1) + "." + time.strftime("%Y%m%d.%H%M", time.localtime())
    else:
        raise RuntimeError("Cannot find version!")


def install_requires():
    """
    Return requires in requirements.txt
    :return:
    """
    try:
        with open("requirements.txt") as f:
            return [line.strip() for line in f.readlines() if line.strip()]
    except OSError:
        return []


def parse_description():
    """
    Parse the description in the README file
    CommandLine:
        pandoc --from=markdown --to=rst --output=README.rst README.md
        python -c "import setup; print(setup.parse_description())"
    """
    from os.path import dirname, join, exists
    readme_fpath = join(dirname(__file__), 'README.rst')
    # This breaks on pip install, so check that it exists.
    if exists(readme_fpath):
        with open(readme_fpath, 'r', encoding="utf-8") as f:
            text = f.read()
        return text
    return ''


from setuptools import setup, find_packages

setup(
    name="pyefun",
    version=get_version('pyefun'),
    packages=find_packages('.'),
    # scripts = ['say_hello.py'],
    # Project uses reStructuredText, so ensure that the docutils get
    # installed or upgraded on the target machine
    install_requires=install_requires(),
    package_data={
        '': ['*.txt', '*.rst', '*.md'],
    },
    # metadata for upload to PyPI
    author="duolabmeng",
    author_email="1715109585@qq.com",
    description="pyefun 为python提供强大且易用的中文函数库，完整的封装了易语言核心支持库所有功能，以及易语言中简单易用的函数",
    license="Apache 2",
    keywords="pyefun 易语言 精易pyefun函数库",
    url="https://github.com/duolabmeng6/pyefun",  # project home page, if any

    long_description=parse_description(),
    long_description_content_type='text/markdown',
)



import unittest

from .dirkBase import *
from .cacheUtil import *
import ubelt as ub


@缓存('name', 'func')  # boilerplate:1
def func():  # boilerplate:2
    data = 'mydata'
    return data  # boilerplate:3


def compute_many_files(dpath):
    for i in range(0):
        fpath = '{}/file{}.txt'.format(dpath, i)
        open(fpath).write('foo' + str(i))


class TestCacheUtil(unittest.TestCase):

    def test_1(self):
        cacher = ub.Cacher(
            fname="mycache",
            depends=ub.hash_data('test'),
            dpath="./cache/",
            appname="my",
            ext=".pkl",
            verbose=3,
            enabled=True,
            hasher='sha1',
            protocol=3,
        )
        print(cacher.get_fpath())
        data = cacher.tryload()
        if data is None:
            myvar1 = 'result of expensive process'
            myvar2 = 'another result'
            data = myvar1, myvar2
            cacher.save(data)
        myvar1, myvar2 = data

        data = func()
        print(data)

        from ubelt.util_cache import Cacher
        from os.path import basename
        # Ensure that some data exists
        known_fpaths = set()
        # cacher = Cacher('versioned_data_v2', depends='1')
        # cacher.ensure(lambda: 'data1')
        # known_fpaths.add(cacher.get_fpath())
        # cacher = Cacher('versioned_data_v2', depends='2')
        # cacher.ensure(lambda: 'data2')
        # known_fpaths.add(cacher.get_fpath())
        # # List previously computed configs for this type
        # cacher = ub.Cacher('versioned_data_v2', depends='2')
        exist_fpaths = set(cacher.existing_versions())
        print('exist_fnames = {!r}'.format(exist_fpaths))

        exist_fnames = list(map(basename, exist_fpaths))
        print('exist_fnames = {!r}'.format(exist_fnames))
        # assert exist_fpaths.issubset(known_fpaths)

    def test_2(self):
        dpath = 取运行目录() + r"\cache"
        # You must specify a directory, unlike in Cacher where it is optional
        self = ub.CacheStamp('name', dpath=dpath, cfgstr='dependencies')
        if self.expired():
            compute_many_files(dpath)

            self.renew()
        assert not self.expired()

    def test_3(self):
        cacher = 缓存("pro", "test")
        data = cacher.读入()
        if data is None:
            myvar1 = 'result of expensive process'
            myvar2 = 'another result'
            data = myvar1, myvar2
            cacher.保存(data)
        myvar1, myvar2 = data
        print(myvar1, myvar2)

        data = func()
        print(data)

        # cacher.清空()

    def test_4(self):
        func标记 = 缓存标记('name2', key='dependencies')
        print("标记", func标记.检查标记())
        if func标记.检查标记():
            print("完成过程", 1)
            func标记.保存标记()
        print("标记", func标记.检查标记())



import unittest
import uuid

from .commonlyUtil import *
from .dirkBase import *


class TestcommonlyUtil(unittest.TestCase):

    def test_1(self):
        pass
        data = 取哈希("abc")
        print(data)

        写到文件("sha1.txt", "abc")
        data = 取文件哈希("sha1.txt")
        print(data)

    def test_2(self):
        pass
        data = 运行命令("ipconfig")
        print(data)

    def test_3(self):
        pass
        data = 取缓存目录("app")
        print(data)

    def test_4(self):
        pass
        # data = 下载文件("https://www.baidu.com/img/PCtm_d9c8750bed0b3c7d089fa7d55720d6cf.png", 取运行目录() + "/1.png")
        # print(data)

        data = 下载文件缓存("https://www.baidu.com/img/PCtm_d9c8750bed0b3c7d089fa7d55720d6cf.png", 取运行目录() + "/1.png")
        print(data)

    def test_5(self):
        选项 = ['ham', 'jam', 'spam', 'eggs', 'cheese', 'bannana']
        分组 = ['protein', 'fruit', 'protein', 'protein', 'dairy', 'fruit']
        data = 字典_分组(选项, 分组)
        print(data)

        items = [1, 2, 39, 900, 1232, 900, 1232, 2, 2, 2, 900]
        data = 字典_统计(items)
        print(data)

        dict_ = {'K': 3, 'dcvs_clip_max': 0.2, 'p': 0.1}
        subdict_ = 字典_取子集(dict_, ['K', 'dcvs_clip_max'])
        print(subdict_)

        dict_ = {1: 'a', 2: 'b', 3: 'c'}
        print(字典_取值(dict_, [1, 2, 3, 4, 5]))

        dict_ = {'a': [1, 2, 3], 'b': [], 'c': [4, 5, 6]}
        newdict = 字典_根据健重建值(len, dict_)
        print(newdict)

        mapping = {'a': 0, 'A': 1, 'b': 1, 'c': 2, 'C': 2, 'd': 3}
        data = 字典_健值交换(mapping)
        print(data)

        mapping = {'a': 0, 'A': 1, 'b': 1, 'c': 2, 'C': 2, 'd': 3}
        data = 字典_健值交换(mapping, False)
        print(data)

    def test2(self):
        auto = 灵活字典()
        print('auto = {!r}'.format(auto))
        auto[0][10][100] = None
        print('auto = {!r}'.format(auto))
        auto[0][1] = 'hello'
        print('auto = {!r}'.format(auto))
        auto[0][100][1] = 'hello'
        print('auto = {!r}'.format(auto))

    def test3(self):
        写到文件(取运行目录() + r"/test/__init__.py", """
def test():
    print("动态导入的包")
        """)

        data = 导入包_从路径(取运行目录() + r"/test/")
        print(data)
        data.test()

    def test4(self):
        items = [0, 0, 0, 1, 2, 3, 3, 0, 12, 2, 9]
        data = 数组_查找重复项(items, 3)
        print(data)
        data = 数组_查找重复项(items, 2)
        print(data)

    def test5(self):
        data = func(1, 2)
        print(data)
        data = func(1, 2)
        print(data)
        data = func(2, 2)
        print(data)
        data = func(2, 2)
        print(data)

        func(1, 2)
        func(1, 2)
        a = MyClass()
        print(a.my_method(1, 2))
        print(a.my_method(1, 2))
        print(a.my_method(2, 2))
        print(a.my_method(2, 2))
        print(a.my_method(1, 2))
        print(a.my_property1)
        print(a.my_property1)
        print(a.my_property2())
        print(a.my_property2())

    def test6(self):
        data = 路径_名字处理('foo.bar')
        print(data)
        # 'foo.bar'
        data = 路径_名字处理('foo.bar', 扩展名='.BAZ')
        print(data)
        # 'foo.BAZ'
        data = 路径_名字处理('foo.bar', 末尾='_')
        print(data)
        # 'foo_.bar'
        data = 路径_名字处理('foo.bar', 前缀='_')
        print(data)
        # '_foo.bar'
        data = 路径_名字处理('foo.bar', 名称='baz')
        print(data)
        # 'baz.bar'
        data = 路径_名字处理('foo.tar.gz', 扩展名='.zip', 不包含点=True)
        print(data)
        # foo.zip
        data = 路径_名字处理('foo.tar.gz', 扩展名='.zip', 不包含点=False)
        print(data)
        # foo.tar.zip
        data = 路径_名字处理('foo.tar.gz', 末尾='_new', 不包含点=True)
        print(data)
        # foo_new.tar.gz

    def test7(self):
        data = 路径_取用户目录()
        print(data)
        # import getpass
        # username = getpass.getuser()
        # data = 路径_取用户目录(username)
        # print(data)

        path = ub.util_path.expanduser('~')
        print(path)

        data = 路径_替换为用户路径(path) == '~'
        print(data)

        data = 路径_替换为用户路径(path + '1')
        print(data)

        data = 路径_替换为用户路径(path + '/1')
        print(data)

        data = 路径_替换为用户路径(path + '/1', '$HOME')
        print(data)

        data = 路径_替换为用户路径('.')
        print(data)
        data = 路径_优化路径(路径_展开路径('~/foo'))
        print(data)
        data = 路径_优化路径(r'c:/123\abc\dbf/dddd')
        print(data)
        data = 路径_展开路径('foo')
        print(data)

    def test666(self):
        data = 路径_合并(r'c://', r"/bb/", r"//cc//", r"dd/aa")
        print(data)
        data = 路径_展开路径('~/foo')
        print(data)



    def test8(self):
        with 临时目录() as self:
            dpath = self.dpath
            print(dpath)
            assert 文件是否存在(dpath)
        assert not 文件是否存在(dpath)

        self = 临时目录()
        dpath = self.初始化()
        print(dpath)
        assert 文件是否存在(dpath)
        self.清理()
        assert not 文件是否存在(dpath)

    def test9(self):
        self = 控制台(获取内容=True)
        with self:
            print('I am captured and printed in stdout')
        print("test3", self.获取内容())

        self = 控制台(获取内容=False)
        with self:
            print('I am captured and printed in stdout')
        print("test2", self.获取内容())

    def test10(self):
        items = '1234567'
        genresult = 分块(items, 分块数量=2)
        print(list(genresult))
        items = '1234567890'
        genresult = 分块(items, 创建块数=1, 创建数量=4)
        print(list(genresult))

    def test10(self):
        data = 取执行文件名()
        self.assertEqual(data, "commonlyUtil.py")

        data = 读环境变量("path")
        print(data)

        data = 写环境变量("aaa", "bbb")
        self.assertEqual(data, "bbb")

        data = 读环境变量("aaa")
        self.assertEqual(data, "bbb")

    def test12(self):
        data = 取主机名()
        print(data)

        # self.assertEqual(data, "commonlyUtil.py")
        data = 转换为IP地址(取主机名())
        # self.assertEqual(data, "commonlyUtil.py")

        print(data)

    def test11(self):
        data = 标准输出("请输入一个数", ":")

        data = 标准输入()
        print(data)

    def test14(self):
        data = 结束()

    def test14(self):
        data = 取uuid()
        print(data)
        data = 取短id()
        print(data)


@内存缓存
def func(a, b):
    print("缓存了")
    return a + b


class MyClass:
    # Memoize a class method, the args are hashed
    @内存缓存方法
    def my_method(self, a, b):
        print("my_method", a, b)
        return a + b

    #
    # Memoize a property: there can be no args,
    @内存缓存属性
    def my_property1(self):
        print("my_property1")
        return 4

    #
    # The property decorator is optional
    def my_property2(self):
        print("my_property2")
        return 5
#



import unittest

from .__init__ import *


class TestClock(unittest.TestCase):

    def test_1(self):
        pass

        @时钟周期事件(时钟周期=1000)
        def 定时任务():
            print("定时任务")
            # return True
            return False

    def test_2(self):
        pass

        def 定时任务():
            print("定时任务")
            # return True
            return False
        时钟(定时任务, 时钟周期=1000)



class 数组:

    def __init__(self, data: list = []):
        self.val = list(data)

    def 加入成员(self, object):
        self.val.append(object)

    def 统计成员次数(self, object):
        return self.val.count(object)

    def 查找成员(self, object):
        return self.val.index(object)

    def 弹出成员(self, index: int = -1):
        return self.val.pop(index)

    def 插入成员(self, object, index: int = -1):
        self.val.insert(index, object)

    def 移除成员(self, object):
        self.val.remove(object)

    def 翻转(self):
        self.val.reverse()

    def 排序(self, **kwargs):
        # cmp=None, key=None, reverse=False
        # cmp -- 可选参数, 如果指定了该参数会使用该参数的方法进行排序。
        # key -- 主要是用来进行比较的元素，只有一个参数，具体的函数的参数就是取自于可迭代对象中，指定可迭代对象中的一个元素来进行排序。
        # reverse -- 排序规则，reverse = True 降序， reverse = False 升序（默认）。
        self.val.sort(**kwargs)

    def 从大到小(self, 下标=0):
        pass
        self.排序(reverse=True, key=lambda d: _func_key(d, 下标))

    def 从小到大(self, 下标=0):
        self.排序(reverse=False, key=lambda d: _func_key(d, 下标))

    def 取所有成员(self):
        return self.val

    def 清空(self):
        self.val = []


def _func_key(d, 下标=0):
    if (type(d) == tuple):
        return d[下标]
    return d



import unittest

from .configUtil import *

class TestConfigUtil(unittest.TestCase):

    def test_1(self):
        pass

        conf = 配置项().加载("""
        [节名称0]

        项目名称0 = value0
        项目名称1 = value1

        [节名称1]

        项目名称2 = value2
        项目名称3 = value3
        """)
        所有节名 = conf.取所有节名()
        print(所有节名)
        for 节名 in 所有节名:
            print(conf.取所有项名和值(节名))
            print(conf.取所有项名(节名))
            for 项名称 in conf.取所有项名(节名):
                print(conf.读配置项(节名, 项名称))

        conf.写配置项("节3", "项目1", "ok")
        conf.写配置项("节3", "项目2", "ok3")
        conf.写配置项("节4", "项目2", "ok3")
        conf.删除配置项("节4", "项目2")
        conf.删除节名("节4", )
        conf.删除节名("节3", )
        data = conf.取数据()
        print(data)
        字典 = conf.取所有结果()
        print(字典)
        print(字典["节名称0"]["项目名称0"])
        print(json到文本(字典))


"""
线程操作类

"""

from .public import *
import threading

from concurrent.futures import ThreadPoolExecutor
from threading import current_thread



@异常处理返回类型逻辑型
def 启动线程(函数名, 参数=(), 跟随主线程结束=False):
    "成功返回线程对象,参数用元组形式传入,返回线程对象,daemon属性为False，主线程结束时会检测该子线程是否结束"
    线程 = threading.Thread(target=函数名, args=参数, daemon=跟随主线程结束)
    线程.start()
    return 线程


class 互斥锁:
    '类似许可证,这个可能会造成死锁'

    def __init__(self):
        self.lock = threading.Lock()

    @异常处理返回类型逻辑型
    def 进入(self):
        self.lock.acquire()

    @异常处理返回类型逻辑型
    def 退出(self):
        self.lock.release()


class 递归锁:
    '类似许可证，互斥锁的升级版，这个不会造成死锁'

    def __init__(self):
        self.lock = threading.RLock()

    @异常处理返回类型逻辑型
    def 进入(self):
        self.lock.acquire()

    @异常处理返回类型逻辑型
    def 退出(self):
        self.lock.release()


class 信号量:
    '设置最大同时运行的线程数'

    def __init__(self, 数量=1):
        self.lock = threading.BoundedSemaphore(数量)

    @异常处理返回类型逻辑型
    def 进入(self):
        self.lock.acquire()

    @异常处理返回类型逻辑型
    def 退出(self):
        self.lock.release()


class 事件锁:
    def __init__(self):
        self.lock = threading.Event()

    @异常处理返回类型逻辑型
    def 通行(self):
        self.lock.set()

    @异常处理返回类型逻辑型
    def 堵塞(self):
        self.lock.clear()

    @异常处理返回类型逻辑型
    def 等待(self):
        '这里会判断事件状态,如果为通行状态则继续向下运行,否则会一直等待'
        self.lock.wait()


class 线程:
    def __init__(self):
        import threading as 线程
        self.__线程 = 线程
        self.__线程列表 = []

    @异常处理返回类型逻辑型
    def 启动线程(self, 函数名, 参数=(), 跟随主线程结束=False):
        "成功返回线程对象,参数用元组形式传入,daemon属性为False，主线程结束时会检测该子线程是否结束"
        线程 = self.__线程.Thread(target=函数名, args=参数, daemon=跟随主线程结束)
        线程.start()
        self.__线程列表.append(线程)
        return 线程

    @异常处理返回类型逻辑型
    def 等待线程结束(self, 最长等待时间=0):
        '顺利结束返回True如果启动线程参数daemon设置为True,则可以设置最长等待时间,超过时间强制结束线程'
        for i in self.__线程列表:
            if 最长等待时间 <= 0:
                i.join()
            else:
                i.join(最长等待时间)
        return True

    @异常处理返回类型逻辑型
    def 取运行中的线程对象(self):
        return self.__线程.enumerate()

    @异常处理返回类型逻辑型
    def 线程是否在运行(self, 线程对象):
        '返回True或False'
        return 线程对象.is_alive()

    @异常处理返回类型逻辑型
    def 取运行的线程数(self):
        '只返回该类创建后使用该类启动线程创建的线程数量'
        for x in self.__线程列表:
            if x.is_alive() == False:
                self.__线程列表.remove(x)
        return len(self.__线程列表)


# 过时的库不用了
# import threadpool
# class 线程池(threadpool.ThreadPool):
#     def __init__(self, 工作线程数量, 队列数量=0):
#         """
#             创建线程池
#             工作线程数量 线程池的线程数量
#             队列数量 为投递任务时队列的数量
#         """
#         super().__init__(工作线程数量, q_size=队列数量, resq_size=0, poll_timeout=5)
#
#     @异常处理返回类型逻辑型
#     def 投递任务(self, 任务函数, 参数, 等待时间=None):
#         """投递一个任务函数到线程池任务队列中。"""
#         requests = threadpool.makeRequests(任务函数, [参数])
#         for req in requests:
#             self.putRequest(req, True, 等待时间)
#
#     def 等待(self):
#         """等待所有任务完成"""
#         self.wait()


def 线程_取活动对象数():
    """
    返回Thread当前活动的对象数。返回的计数等于所返回列表的长度
    """
    return threading.active_count()


def 线程_取当前线程():
    """
    返回与Thread调用方的控制线程相对应的当前对象。如果未通过threading模块创建调用者的控制 线程，则返回功能有限的虚拟线程对象。
    """
    return threading.current_thread()


def 线程_取线程标识符():
    """
    返回当前线程的“线程标识符”。这是一个非零整数。它的值没有直接的意义。它旨在用作魔术Cookie，例如用于索引线程特定数据的字典。当一个线程退出并创建另一个线程时，线程标识符可以被回收。
    """

    return threading.get_ident()


def 线程_取所有活动对象():
    """
返回Thread当前所有活动对象的列表。该列表包括守护线程，由创建的伪线程对象 current_thread()和主线程。它不包括终止的线程和尚未启动的线程。
    """
    return threading.enumerate()


def 线程_取主线程():
    """
 返回主要Thread对象。在正常情况下，主线程是启动Python解释器的线程。

  """
    return threading.main_thread()







def 取当前线程名称():
    return current_thread().name





class 线程池(ThreadPoolExecutor):
    """
    当有大量并发任务需要处理时，再使用传统的多线程就会造成大量的资源创建销毁导致服务器效率的下降。这时候，线程池就派上用场了。线程池技术为线程创建、销毁的开销问题和系统资源不足问题提供了很好的解决方案。

    用法

    def 线程初始化(data):
        print("初始化", data, 取当前线程名称())

    互斥锁 = 互斥锁()
    def 任务函数(i):
        time.sleep(1)
        互斥锁.进入()
        print(i)
        互斥锁.退出()
        return "返回参数" + str(i)

    def 任务完成(future):
        print("当前线程", 取当前线程名称())
        print("future", future.result())

    任务池 = 线程池(4, "pyefun", 线程初始化, [0])

    for url in range(10):
        future = 任务池.投递任务(任务函数, url)
        任务池.设置任务结束回调函数(future, 任务完成)

    任务池.等待()
    """

    def __init__(self, 最大线程数量, 线程名称前缀='',
                 线程初始化函数=None, 初始化函数参数=()):
        """
            创建线程池

            :param 最大线程数量:
            :param 线程名称前缀:
            :param 线程初始化函数:
            :param 初始化函数参数:
        """
        super().__init__(最大线程数量, 线程名称前缀, 线程初始化函数, 初始化函数参数)

    def 投递任务(self, *任务函数, **传入参数):
        """
        投递任务

        :param 任务函数:
        :param 传入参数:
        :return: Future 对象可以 设置任务结束回到函数
        """
        Future = self.submit(*任务函数, **传入参数)
        return Future

    def 设置任务结束回调函数(self, future, 回到函数):
        """
        投递任务返回的对象

        def 回到函数(线程返回的结果):
            print("当前线程", current_thread().name)
            print("线程返回的结果", future.result())

        """
        future.add_done_callback(回到函数)

    def 等待(self):
        self.shutdown(True)

    def 批量投递任务(self, 任务函数, *任务参数数组, 超时=None, chunksize=1):
        """
        批量投递任务 不能设置回到函数
        :param 任务函数:
        :param 任务参数数组:
        :param 超时:
        :param chunksize:
        :return:
        """
        return self.map(任务函数, *任务参数数组, timeout=超时, chunksize=chunksize)







